package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamCaptureQueueStatus;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureCommonService;
import cn.com.qmth.examcloud.core.oe.student.face.service.bean.CallType;

/**
 * 
 * @author  	chenken
 * @date    	2018年12月18日 下午4:34:46
 * @company 	QMTH
 * @description ExamCaptureCommonServiceImpl.java
 */
@Service("examCaptureCommonService")
public class ExamCaptureCommonServiceImpl implements ExamCaptureCommonService {

	private static final Logger log = LoggerFactory.getLogger(ExamCaptureCommonServiceImpl.class);
	
	@Autowired
	private RedisTemplate<String,Object> redisTemplate;
	
	@Autowired
	private ExamCaptureQueueRepo examCaptureQueueRepo;
	
	@Override
	public boolean checkLockIsExistsAndDispose(ExamCaptureQueueEntity examCaptureQueue,CallType callType){
		if (callType!=CallType.FAILED_DISPOSE)
		//非失败处理且存在交卷锁
		if(redisTemplate.hasKey(Constants.EXAM_CAPTURE_PHOTO_LOCK_PREFIX+examCaptureQueue.getExamRecordDataId()) ||
				!redisTemplate.hasKey(Constants.OE_STUDENT_EXAM_SESSION_PREFIX+examCaptureQueue.getStudentId())){
			examCaptureQueueRepo.delete(examCaptureQueue);//删除队列中的记录
			return true;
		}
		return false;
	}
	
	private void dispose(ExamCaptureQueueEntity examCaptureQueue){
		if(examCaptureQueue.getStatus() == ExamCaptureQueueStatus.PROCESS_FACE_COMPARE_FAILED){
			/**
			 * 存在锁且记录已经失败过，将错误次数设置为4，不在此时处理，交给失败任务处理
			 */
			int errorNum = examCaptureQueue.getErrorNum();
			examCaptureQueue.setErrorNum(errorNum<4?4:errorNum);
			examCaptureQueueRepo.save(examCaptureQueue);
		}else{
			log.info("redis中存在交卷锁或考试会话不存在,抓拍记录被删除:"+examCaptureQueue.getId());
			examCaptureQueueRepo.delete(examCaptureQueue);//删除队列中的记录
		}
	}

}
